"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.graphUpdateObject = void 0;

var _actionType = require("./actionType");

var _terminusdbClient = _interopRequireDefault(require("@terminusdb/terminusdb-client"));

var _elementsName = require("../utils/elementsName");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var graphUpdateObject = function graphUpdateObject() {
  var newNodesList = new Map();
  var newPropertiesList = new Map();
  var deleteNodesList = new Map();
  var deletePropertiesList = new Map();
  var updateChoiceList = new Map();
  var updateTriple = new Map();
  /*
  * for the interface I can change the parentship from child to parent
  */

  var changeParentList = new Map();

  var addParent = function addParent(currentNode, newNode, isChoiceClass) {
    if (currentNode.type === _elementsName.CLASS_TYPE_NAME.SCHEMA_GROUP) {
      if (currentNode.name === _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASSES) {
        newNode.parent = ['Document'];
        newNode.parents = [];
        newNode.type = _elementsName.CLASS_TYPE_NAME.DOCUMENT_CLASS;
      } else {
        newNode.parent = [];
        newNode.parents = [];
        var nodeType = _elementsName.CLASS_TYPE_NAME.OBJECT_CLASS;

        if (isChoiceClass) {
          nodeType = _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS;
          newNode.choices = [];
        }

        newNode.type = nodeType;
      }
    } else {
      newNode.parent = [currentNode.id];
      newNode.parents = [currentNode.name];
      newNode.type = currentNode.type;
    }
  };

  var updateChoicesList = function updateChoicesList(choiceClass) {
    if (choiceClass.newElement === true) return;
    updateChoiceList.set(choiceClass.name, choiceClass);
  };
  /*
  * name is internal unique reference 
  */


  var updateTripleElement = function updateTripleElement(propName, propValue, currentElement) {
    /*
    * I don't need to add a triple change because this is a new node
    */
    if (currentElement.newElement === true) return;
    var values = {};

    if (updateTriple.has(currentElement.name)) {
      values = updateTriple.get(currentElement.name);
      values[propName] = propValue;
    } else {
      values[propName] = propValue;
      updateTriple.set(currentElement.name, values);
    }
    /*
    * if the element is a property
    */


    if (currentElement.domain) {
      values['domain'] = currentElement.domain;
    }
  };
  /*
  * newNodeParent : parent of the new node 
  * newNodeChild  : child of the new node
  */


  var addNodeToTree = function addNodeToTree(newNodeParent) {
    var newNodeChild = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var isChoiceClass = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var newName = "CLASS_".concat(new Date().getTime());
    var elementModel = {
      name: newName,
      id: "",
      label: "NEW NODE",
      comment: "",
      newElement: true,
      children: [],
      "abstract": false
    };
    addParent(newNodeParent, elementModel, isChoiceClass);
    newNodesList.set(newName, elementModel);

    if (newNodeChild) {
      /*
      * add the parent relationship to the child node
      */
      changeNodeParent(newNodeChild.name, newName, _actionType.ADD_PARENT);
      newNodeChild.parents.push(newName);
    }

    return elementModel;
  };

  var changeNodeParent = function changeNodeParent(childName, parentName, action) {
    var parentsList = [];

    if (changeParentList.has(childName)) {
      parentsList = changeParentList.get(childName);
    } else {
      changeParentList.set(childName, parentsList);
    }

    parentsList.push({
      childNode: childName,
      parentNode: parentName,
      action: action
    });
  };

  var removeNode = function removeNode(nodeObj) {
    if (nodeObj.newElement === true) {
      newNodesList["delete"](nodeObj.name);
    } else {
      deleteNodesList.set(nodeObj.name, nodeObj);
    }
  };
  /*
  * add a new property when I'll save 
  * I have to replace the domain if the property is for a new node 
  */


  var addPropertyToClass = function addPropertyToClass(domainNodeName, propertyType, propertyRange) {
    var newName = "PROPERTY_".concat(new Date().getTime());
    var elementModel = {
      name: newName,
      id: "",
      label: "NEW PROPERTY",
      comment: "",
      type: propertyType,
      newElement: true,
      range: propertyRange,
      domain: domainNodeName
    };
    newPropertiesList.set(newName, elementModel);
    return elementModel;
  };

  var removePropertyToClass = function removePropertyToClass(propertyObject) {
    /*
    * if the property is only local (not saved in the db) 
    * I can remove it from the list 
    */
    if (propertyObject.newElement === true) {
      newPropertiesList["delete"](propertyObject.name);
    } else {
      deletePropertiesList.set(propertyObject.name, propertyObject);
    }
  };
  /*
  * In new node we use a internal name for reference 
  */


  var getPropertyObjForSave = function getPropertyObjForSave(propertyObj) {
    var copyNode = JSON.parse(JSON.stringify(propertyObj));
    copyNode.description = copyNode.comment;
    copyNode['domain'] = getRealId(copyNode.domain);

    if (propertyObj.type === _elementsName.PROPERTY_TYPE_NAME.OBJECT_PROPERTY) {
      copyNode['range'] = getRealId(copyNode.range);
    }

    copyNode.id = "scm:".concat(propertyObj.id);
    return copyNode;
  };

  var getRealId = function getRealId(elementName) {
    //scm:bike_parent_003
    if (newNodesList.has(elementName)) {
      return "scm:".concat(newNodesList.get(elementName).id);
    }

    return elementName; //const arr = elementName.split("#")
    //return `scm:${arr[1]}`	
  };

  var formatChoiceListForWoql = function formatChoiceListForWoql(choicelist) {
    var choices = [];
    choicelist.forEach(function (item) {
      var choiceArr = ["scm:".concat(item.id), item.label, item.comment];
      choices.push(choiceArr);
    });
    return choices;
  };

  var savedObjectToWOQL = function savedObjectToWOQL() {
    var WOQL = _terminusdbClient["default"].WOQL;
    var andValues = [];
    newNodesList.forEach(function (node, key) {
      /*if(node.id===undefined){
      	alert(`The node ${node.name}`)
      }*/
      if (node.type !== _elementsName.CLASS_TYPE_NAME.CHOICE_CLASS) {
        var newNode = {
          id: node.id,
          label: node.label,
          description: node.comment,
          parent: node.parent,
          "abstract": node["abstract"]
        };
        andValues.push(WOQL.insert_class_data(newNode));
      } else {
        var choices = formatChoiceListForWoql(node.choices);
        andValues.push(WOQL.generateChoiceList(node.id, node.label, node.comment, choices));
      }
    });
    newPropertiesList.forEach(function (property, key) {
      var propertyObj = getPropertyObjForSave(property);
      andValues.push(WOQL.insert_property_data(propertyObj));
    });
    changeParentList.forEach(function (parentActArr, key) {
      parentActArr.forEach(function (item, key) {
        var idChild = getRealId(item.childNode);
        var idParent = getRealId(item.parentNode);

        switch (item.action) {
          case _actionType.ADD_PARENT:
            andValues.push(WOQL.add_quad(idChild, "subClassOf", idParent, "schema/main"));
            break;

          case _actionType.REMOVE_PARENT:
            andValues.push(WOQL.delete_quad(idChild, "subClassOf", idParent, "schema/main"));
        }
      });
    });
    updateTriple.forEach(function (valuesObject, subject) {
      var subjectId = getRealId(subject);

      for (var vname in valuesObject) {
        if (vname === "domain") continue;

        if (['min', 'max', 'cardinality'].indexOf(vname) > -1) {
          andValues.push(updateCardinality(WOQL, subjectId, vname, valuesObject[vname], valuesObject['domain']));
        } else {
          if (vname === 'abstract') {
            andValues.push(updateAbstract(WOQL, subjectId, valuesObject[vname]));
          } else {
            andValues.push(WOQL.update_quad(subjectId, vname, valuesObject[vname], 'schema/main'));
          }
        }
      }
    });
    deleteNodesList.forEach(function (nodeObj) {
      var varName = "v:".concat(nodeObj.id);
      andValues.push(WOQL.delete_class(nodeObj.name, null, varName));
    });
    deletePropertiesList.forEach(function (proObj) {
      var varName = "v:".concat(proObj.id);
      andValues.push(WOQL.delete_property(proObj.name, null, varName));
    });
    /*
    * if I change somethingin the choicelist I have to remove and add again
    */

    updateChoiceList.forEach(function (choiceObj, choiceName) {
      var choiceId = getRealId(choiceName);
      var choices = formatChoiceListForWoql(choiceObj.choices);
      andValues.push(WOQL.updateChoiceList(choiceId, choiceObj.label, choiceObj.comment, choices));
    });
    if (andValues.length === 0) return undefined;
    var query = WOQL.and.apply(WOQL, andValues);
    return query;
  };

  var updateAbstract = function updateAbstract(WOQL, subjectId, value) {
    if (value === true) {
      return WOQL.update_quad(subjectId, 'system:tag', 'system:abstract', 'schema/main');
    }

    return WOQL.opt(WOQL.quad(subjectId, 'system:tag', "system:abstract", "schema/main").delete_quad(subjectId, 'system:tag', "system:abstract", "schema/main"));
  };
  /*
  *to be review if I have both and I like to update only one 
  *maybe I don't have to remove all the retriction
  */


  var updateCardinality = function updateCardinality(WOQL, subjectId, vname, value, domain) {
    var cardType = {
      max: 'owl:maxCardinality',
      min: 'owl:minCardinality'
    };
    var cardName = subjectId + '_' + vname + '_' + value;
    var cardTypeName = cardType[vname];
    var domainId = getRealId(domain);
    return WOQL.and(WOQL.opt(WOQL.quad("v:Restriction", "owl:onProperty", subjectId, "schema/main").delete_quad("v:Restriction", "owl:onProperty", subjectId, "schema/main")), WOQL.add_quad(cardName, "type", "owl:Restriction", "schema/main").add_quad(cardName, "owl:onProperty", subjectId, "schema/main").add_quad(cardName, cardTypeName, WOQL.literal(value, "xsd:nonNegativeInteger"), "schema/main").add_quad(domainId, "subClassOf", cardName, "schema/main"));
  };

  return {
    updateChoicesList: updateChoicesList,
    addNodeToTree: addNodeToTree,
    changeNodeParent: changeNodeParent,
    addPropertyToClass: addPropertyToClass,
    removePropertyToClass: removePropertyToClass,
    savedObjectToWOQL: savedObjectToWOQL,
    removeNode: removeNode,
    updateTripleElement: updateTripleElement
  };
}; //new node 
//new property

/*

*/
//update triple (value to a prop)
//cardinality (change chardinality value)
//if they change locally like add new node remove new node 
//if they remove or change some


exports.graphUpdateObject = graphUpdateObject;